@@ -19,7 +19,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include "libtrace/kbuffer.h"
+#include <traceevent/kbuffer.h>
 #include "ras-aer-handler.h"
 #include "ras-record.h"
 #include "ras-logger.h"
@@ -35,6 +35,7 @@ static const char *aer_cor_errors[32] = {
 	[8]  = "RELAY_NUM Rollover",
 	[12] = "Replay Timer Timeout",
 	[13] = "Advisory Non-Fatal",
+	[14] = "Corrected Internal Error",
 };
 
 /* bit field meaning for uncorrectable error */
@@ -55,8 +56,8 @@ static const char *aer_uncor_errors[32] = {
 #define BUF_LEN	1024
 
 int ras_aer_event_handler(struct trace_seq *s,
-			 struct pevent_record *record,
-			 struct event_format *event, void *context)
+			  struct tep_record *record,
+			  struct tep_event *event, void *context)
 {
 	int len;
 	unsigned long long severity_val;
@@ -67,9 +68,11 @@ int ras_aer_event_handler(struct trace_seq *s,
 	struct tm *tm;
 	struct ras_aer_event ev;
 	char buf[BUF_LEN];
+#ifdef HAVE_AMP_NS_DECODE
 	char ipmi_add_sel[105];
 	uint8_t sel_data[5];
-	int seg, bus, dev, fn;
+	int seg, bus, dev, fn, rc;
+#endif
 
 	/*
 	 * Newer kernels (3.10-rc1 or upper) provide an uptime clock.
@@ -81,7 +84,7 @@ int ras_aer_event_handler(struct trace_seq *s,
 	 */
 
 	if (ras->use_uptime)
-		now = record->ts/user_hz + ras->uptime_diff;
+		now = record->ts / user_hz + ras->uptime_diff;
 	else
 		now = time(NULL);
 
@@ -91,16 +94,16 @@ int ras_aer_event_handler(struct trace_seq *s,
 			 "%Y-%m-%d %H:%M:%S %z", tm);
 	trace_seq_printf(s, "%s ", ev.timestamp);
 
-	ev.dev_name = pevent_get_field_raw(s, event, "dev_name",
-					   record, &len, 1);
+	ev.dev_name = tep_get_field_raw(s, event, "dev_name",
+					record, &len, 1);
 	if (!ev.dev_name)
 		return -1;
 	trace_seq_printf(s, "%s ", ev.dev_name);
 
-	if (pevent_get_field_val(s,  event, "status", record, &status_val, 1) < 0)
+	if (tep_get_field_val(s,  event, "status", record, &status_val, 1) < 0)
 		return -1;
 
-	if (pevent_get_field_val(s, event, "severity", record, &severity_val, 1) < 0)
+	if (tep_get_field_val(s, event, "severity", record, &severity_val, 1) < 0)
 		return -1;
 
 	/* Fills the error buffer. If it is a correctable error then use the
@@ -112,14 +115,14 @@ int ras_aer_event_handler(struct trace_seq *s,
 		bitfield_msg(buf, sizeof(buf), aer_uncor_errors, 32, 0, 0, status_val);
 	ev.msg = buf;
 
-	if (pevent_get_field_val(s, event, "tlp_header_valid",
-				record, &val, 1) < 0)
+	if (tep_get_field_val(s, event, "tlp_header_valid",
+			      record, &val, 1) < 0)
 		return -1;
 
 	ev.tlp_header_valid = val;
 	if (ev.tlp_header_valid) {
-		ev.tlp_header = pevent_get_field_raw(s, event, "tlp_header",
-						     record, &len, 1);
+		ev.tlp_header = tep_get_field_raw(s, event, "tlp_header",
+						  record, &len, 1);
 		snprintf((buf + strlen(ev.msg)), BUF_LEN - strlen(ev.msg),
 			 " TLP Header: %08x %08x %08x %08x",
 			 ev.tlp_header[0], ev.tlp_header[1],
@@ -132,19 +135,27 @@ int ras_aer_event_handler(struct trace_seq *s,
 	switch (severity_val) {
 	case HW_EVENT_AER_UNCORRECTED_NON_FATAL:
 		ev.error_type = "Uncorrected (Non-Fatal)";
+#ifdef HAVE_AMP_NS_DECODE
 		sel_data[0] = 0xca;
+#endif
 		break;
 	case HW_EVENT_AER_UNCORRECTED_FATAL:
 		ev.error_type = "Uncorrected (Fatal)";
+#ifdef HAVE_AMP_NS_DECODE
 		sel_data[0] = 0xca;
+#endif
 		break;
 	case HW_EVENT_AER_CORRECTED:
 		ev.error_type = "Corrected";
+#ifdef HAVE_AMP_NS_DECODE
 		sel_data[0] = 0xbf;
+#endif
 		break;
 	default:
 		ev.error_type = "Unknown severity";
+#ifdef HAVE_AMP_NS_DECODE
 		sel_data[0] = 0xbf;
+#endif
 	}
 	trace_seq_puts(s, ev.error_type);
 
@@ -176,10 +187,12 @@ int ras_aer_event_handler(struct trace_seq *s,
 	sel_data[4] = (((dev & 0x1f) << 3) | (fn & 0x7));
 
 	sprintf(ipmi_add_sel,
-	  "ipmitool raw 0x0a 0x44 0x00 0x00 0xc0 0x00 0x00 0x00 0x00 0x3a 0xcd 0x00 0xc0 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x",
+		"ipmitool raw 0x0a 0x44 0x00 0x00 0xc0 0x00 0x00 0x00 0x00 0x3a 0xcd 0x00 0xc0 0x%02x 0x%02x 0x%02x 0x%02x 0x%02x",
 	  sel_data[0], sel_data[1], sel_data[2], sel_data[3], sel_data[4]);
 
-	system(ipmi_add_sel);
+	rc = system(ipmi_add_sel);
+	if (rc)
+		log(SYSLOG, LOG_WARNING, "Failed to execute ipmitool\n");
 #endif
 
 	return 0;
