@@ -20,10 +20,10 @@
 #define __RAS_DISKERROR_HANDLER_H
 
 #include "ras-events.h"
-#include "libtrace/event-parse.h"
+#include <traceevent/event-parse.h>
 
 int ras_diskerror_event_handler(struct trace_seq *s,
-				struct pevent_record *record,
-				struct event_format *event, void *context);
+				struct tep_record *record,
+				struct tep_event *event, void *context);
 
 #endif
