@@ -28,8 +28,8 @@
 #include <sys/poll.h>
 #include <signal.h>
 #include <sys/signalfd.h>
-#include "libtrace/kbuffer.h"
-#include "libtrace/event-parse.h"
+#include <traceevent/kbuffer.h>
+#include <traceevent/event-parse.h>
 #include "ras-mc-handler.h"
 #include "ras-aer-handler.h"
 #include "ras-non-standard-handler.h"
@@ -39,9 +39,12 @@
 #include "ras-devlink-handler.h"
 #include "ras-diskerror-handler.h"
 #include "ras-memory-failure-handler.h"
+#include "ras-cxl-handler.h"
 #include "ras-record.h"
 #include "ras-logger.h"
 #include "ras-page-isolation.h"
+#include "ras-cpu-isolation.h"
+#include "trigger.h"
 
 /*
  * Polling time, if read() doesn't block. Currently, trace_pipe_raw never
@@ -57,13 +60,20 @@
 	#define ENDIAN KBUFFER_ENDIAN_BIG
 #endif
 
+extern char *choices_disable;
+
+static const struct event_trigger event_triggers[] = {
+	{ "mc_event", &mc_event_trigger_setup },
+	{ "memory_failure_event", &mem_fail_event_trigger_setup },
+};
+
 static int get_debugfs_dir(char *tracing_dir, size_t len)
 {
 	FILE *fp;
 	char line[MAX_PATH + 1 + 256];
 	char *p, *type, *dir;
 
-	fp = fopen("/proc/mounts","r");
+	fp = fopen("/proc/mounts", "r");
 	if (!fp) {
 		log(ALL, LOG_INFO, "Can't open /proc/mounts");
 		return errno;
@@ -91,7 +101,7 @@ static int get_debugfs_dir(char *tracing_dir, size_t len)
 			tracing_dir[len - 1] = '\0';
 			return 0;
 		}
-	} while(1);
+	} while (1);
 
 	fclose(fp);
 	log(ALL, LOG_INFO, "Can't find debugfs\n");
@@ -136,7 +146,7 @@ static int get_tracing_dir(struct ras_events *ras)
 	strcat(ras->tracing, "/tracing");
 	if (has_instances) {
 		strcat(ras->tracing, "/instances/" TOOL_NAME);
-		rc = mkdir(ras->tracing, S_IRWXU);
+		rc = mkdir(ras->tracing, 0700);
 		if (rc < 0 && errno != EEXIST) {
 			log(ALL, LOG_INFO,
 			    "Unable to create " TOOL_NAME " instance at %s\n",
@@ -147,6 +157,19 @@ static int get_tracing_dir(struct ras_events *ras)
 	return 0;
 }
 
+static int is_disabled_event(char *group, char *event)
+{
+	char ras_event_name[MAX_PATH + 1];
+
+	snprintf(ras_event_name, sizeof(ras_event_name), "%s:%s",
+		 group, event);
+
+	if (choices_disable && strlen(choices_disable) != 0 && strstr(choices_disable, ras_event_name)) {
+		return 1;
+	}
+	return 0;
+}
+
 /*
  * Tracing enable/disable code
  */
@@ -156,6 +179,8 @@ static int __toggle_ras_mc_event(struct ras_events *ras,
 	int fd, rc;
 	char fname[MAX_PATH + 1];
 
+	enable = is_disabled_event(group, event) ? 0 : 1;
+
 	snprintf(fname, sizeof(fname), "%s%s:%s\n",
 		 enable ? "" : "!",
 		 group, event);
@@ -167,7 +192,7 @@ static int __toggle_ras_mc_event(struct ras_events *ras,
 		return errno;
 	}
 
-	rc = write(fd, fname,strlen(fname));
+	rc = write(fd, fname, strlen(fname));
 	if (rc < 0) {
 		log(ALL, LOG_WARNING, "Can't write to set_event\n");
 		close(fd);
@@ -230,18 +255,45 @@ int toggle_ras_mc_event(int enable)
 #endif
 
 #ifdef HAVE_DISKERROR
+#ifdef HAVE_BLK_RQ_ERROR
+	rc |= __toggle_ras_mc_event(ras, "block", "block_rq_error", enable);
+#else
 	rc |= __toggle_ras_mc_event(ras, "block", "block_rq_complete", enable);
 #endif
+#endif
 
 #ifdef HAVE_MEMORY_FAILURE
 	rc |= __toggle_ras_mc_event(ras, "ras", "memory_failure_event", enable);
 #endif
 
+#ifdef HAVE_CXL
+	rc |= __toggle_ras_mc_event(ras, "cxl", "cxl_poison", enable);
+	rc |= __toggle_ras_mc_event(ras, "cxl", "cxl_aer_uncorrectable_error", enable);
+	rc |= __toggle_ras_mc_event(ras, "cxl", "cxl_aer_correctable_error", enable);
+	rc |= __toggle_ras_mc_event(ras, "cxl", "cxl_overflow", enable);
+	rc |= __toggle_ras_mc_event(ras, "cxl", "cxl_generic_event", enable);
+	rc |= __toggle_ras_mc_event(ras, "cxl", "cxl_general_media", enable);
+	rc |= __toggle_ras_mc_event(ras, "cxl", "cxl_dram", enable);
+	rc |= __toggle_ras_mc_event(ras, "cxl", "cxl_memory_module", enable);
+#endif
+
 free_ras:
 	free(ras);
 	return rc;
 }
 
+static void setup_event_trigger(char *event)
+{
+	struct event_trigger trigger;
+
+	for (int i = 0; i < ARRAY_SIZE(event_triggers); i++) {
+		trigger = event_triggers[i];
+		if (!strcmp(event, trigger.name))
+			trigger.setup();
+	}
+}
+
+#ifndef HAVE_BLK_RQ_ERROR
 /*
  * Set kernel filter. libtrace doesn't provide an API for setting filters
  * in kernel, we have to implement it here.
@@ -259,7 +311,7 @@ static int filter_ras_mc_event(struct ras_events *ras, char *group, char *event,
 		return errno;
 	}
 
-	rc = write(fd, filter_str ,strlen(filter_str));
+	rc = write(fd, filter_str, strlen(filter_str));
 	if (rc < 0) {
 		log(ALL, LOG_WARNING, "Can't write to filter file\n");
 		close(fd);
@@ -273,12 +325,13 @@ static int filter_ras_mc_event(struct ras_events *ras, char *group, char *event,
 
 	return 0;
 }
+#endif
 
 /*
  * Tracing read code
  */
 
-static int get_pagesize(struct ras_events *ras, struct pevent *pevent)
+static int get_pagesize(struct ras_events *ras, struct tep_handle *pevent)
 {
 	int fd, len, page_size = 4096;
 	char buf[page_size];
@@ -290,21 +343,18 @@ static int get_pagesize(struct ras_events *ras, struct pevent *pevent)
 	len = read(fd, buf, page_size);
 	if (len <= 0)
 		goto error;
-	if (pevent_parse_header_page(pevent, buf, len, sizeof(long)))
+	if (tep_parse_header_page(pevent, buf, len, sizeof(long)))
 		goto error;
 
-	page_size = pevent->header_page_data_offset + pevent->header_page_data_size;
-
 error:
 	close(fd);
 	return page_size;
-
 }
 
 static void parse_ras_data(struct pthread_data *pdata, struct kbuffer *kbuf,
 			   void *data, unsigned long long time_stamp)
 {
-	struct pevent_record record;
+	struct tep_record record;
 	struct trace_seq s;
 
 	record.ts = time_stamp;
@@ -319,7 +369,11 @@ static void parse_ras_data(struct pthread_data *pdata, struct kbuffer *kbuf,
 
 	/* TODO - logging */
 	trace_seq_init(&s);
-	pevent_print_event(pdata->ras->pevent, &s, &record);
+	tep_print_event(pdata->ras->pevent, &s, &record,
+			"%16s-%-5d [%03d] %s %6.1000d %s %s",
+			TEP_PRINT_COMM, TEP_PRINT_PID, TEP_PRINT_CPU,
+			TEP_PRINT_LATENCY, TEP_PRINT_TIME, TEP_PRINT_NAME,
+			TEP_PRINT_INFO);
 	trace_seq_do_printf(&s);
 	printf("\n");
 	fflush(stdout);
@@ -328,7 +382,7 @@ static void parse_ras_data(struct pthread_data *pdata, struct kbuffer *kbuf,
 
 static int get_num_cpus(struct ras_events *ras)
 {
-	return sysconf(_SC_NPROCESSORS_CONF);
+	return sysconf(_SC_NPROCESSORS_ONLN);
 #if 0
 	char fname[MAX_PATH + 1];
 	int num_cpus = 0;
@@ -351,10 +405,37 @@ static int get_num_cpus(struct ras_events *ras)
 #endif
 }
 
-static int read_ras_event_all_cpus(struct pthread_data *pdata,
-				   unsigned n_cpus)
+static int set_buffer_percent(struct ras_events *ras, int percent)
 {
-	unsigned size;
+	char buf[16];
+	ssize_t size;
+	int res = 0;
+	int fd;
+
+	fd = open_trace(ras, "buffer_percent", O_WRONLY);
+	if (fd >= 0) {
+		/* For the backward compatibility to the old kernels, do not return
+		 * if fail to set the buffer_percent.
+		 */
+		snprintf(buf, sizeof(buf), "%d", percent);
+		size = write(fd, buf, strlen(buf));
+		if (size <= 0) {
+			log(TERM, LOG_WARNING, "can't write to buffer_percent\n");
+			res = -1;
+		}
+		close(fd);
+	} else {
+		log(TERM, LOG_WARNING, "Can't open buffer_percent\n");
+		res = -1;
+	}
+
+	return res;
+}
+
+static int read_ras_event_all_cpus(struct pthread_data *pdata,
+				   unsigned int n_cpus)
+{
+	ssize_t size;
 	unsigned long long time_stamp;
 	void *data;
 	int ready, i, count_nready;
@@ -385,6 +466,16 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 		return -ENOMEM;
 	}
 
+	/* Fix for poll() on the per_cpu trace_pipe and trace_pipe_raw blocks
+	 * indefinitely with the default buffer_percent in the kernel trace system,
+	 * which is introduced by the following change in the kernel.
+	 * https://lore.kernel.org/all/20221020231427.41be3f26@gandalf.local.home/T/#u.
+	 * Set buffer_percent to 0 so that poll() will return immediately
+	 * when the trace data is available in the ras per_cpu trace pipe_raw
+	 */
+	if (set_buffer_percent(pdata[0].ras, 0))
+		log(TERM, LOG_WARNING, "Set buffer_percent failed\n");
+
 	for (i = 0; i < (n_cpus + 1); i++)
 		fds[i].fd = -1;
 
@@ -393,7 +484,7 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 
 		/* FIXME: use select to open for all CPUs */
 		snprintf(pipe_raw, sizeof(pipe_raw),
-			"per_cpu/cpu%d/trace_pipe_raw", i);
+			 "per_cpu/cpu%d/trace_pipe_raw", i);
 
 		fds[i].fd = open_trace(pdata[0].ras, pipe_raw, O_RDONLY);
 		if (fds[i].fd < 0) {
@@ -420,6 +511,10 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 	if (pdata[0].ras->record_events) {
 		if (ras_mc_event_opendb(pdata[0].cpu, pdata[0].ras))
 			goto error;
+#ifdef HAVE_NON_STANDARD
+		if (ras_ns_add_vendor_tables(pdata[0].ras))
+			log(TERM, LOG_ERR, "Can't add vendor table\n");
+#endif
 	}
 
 	do {
@@ -439,7 +534,7 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 			    fdsiginfo.ssi_signo == SIGTERM ||
 			    fdsiginfo.ssi_signo == SIGHUP ||
 			    fdsiginfo.ssi_signo == SIGQUIT) {
-				log(TERM, LOG_INFO, "Recevied signal=%d\n",
+				log(TERM, LOG_INFO, "Received signal=%d\n",
 				    fdsiginfo.ssi_signo);
 				goto  cleanup;
 			} else {
@@ -473,6 +568,11 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 				kbuffer_load_subbuffer(kbuf, page);
 
 				while ((data = kbuffer_read_event(kbuf, &time_stamp))) {
+					if (kbuffer_curr_size(kbuf) < 0) {
+						log(TERM, LOG_ERR, "invalid kbuf data, discard\n");
+						break;
+					}
+
 					parse_ras_data(&pdata[i],
 						       kbuf, data, time_stamp);
 
@@ -504,8 +604,12 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 	    "Old kernel detected. Stop listening and fall back to pthread way.\n");
 
 cleanup:
-	if (pdata[0].ras->record_events)
+	if (pdata[0].ras->record_events) {
+#ifdef HAVE_NON_STANDARD
+		ras_ns_finalize_vendor_tables();
+#endif
 		ras_mc_event_closedb(pdata[0].cpu, pdata[0].ras);
+	}
 
 error:
 	kbuffer_free(kbuf);
@@ -593,19 +697,32 @@ static void *handle_ras_events_cpu(void *priv)
 
 	log(TERM, LOG_INFO, "Listening to events on cpu %d\n", pdata->cpu);
 	if (pdata->ras->record_events) {
+		pthread_mutex_lock(&pdata->ras->db_lock);
 		if (ras_mc_event_opendb(pdata->cpu, pdata->ras)) {
+			pthread_mutex_unlock(&pdata->ras->db_lock);
 			log(TERM, LOG_ERR, "Can't open database\n");
 			close(fd);
 			kbuffer_free(kbuf);
 			free(page);
 			return 0;
 		}
+#ifdef HAVE_NON_STANDARD
+		if (ras_ns_add_vendor_tables(pdata->ras))
+			log(TERM, LOG_ERR, "Can't add vendor table\n");
+#endif
+		pthread_mutex_unlock(&pdata->ras->db_lock);
 	}
 
 	read_ras_event(fd, pdata, kbuf, page);
 
-	if (pdata->ras->record_events)
+	if (pdata->ras->record_events) {
+		pthread_mutex_lock(&pdata->ras->db_lock);
+#ifdef HAVE_NON_STANDARD
+		ras_ns_finalize_vendor_tables();
+#endif
 		ras_mc_event_closedb(pdata->cpu, pdata->ras);
+		pthread_mutex_unlock(&pdata->ras->db_lock);
+	}
 
 	close(fd);
 	kbuffer_free(kbuf);
@@ -622,7 +739,7 @@ static int select_tracing_timestamp(struct ras_events *ras)
 	int fd, rc;
 	time_t uptime, now;
 	size_t size;
-	unsigned j1;
+	unsigned int j1;
 	char buf[4096];
 
 	/* Check if uptime is supported (kernel 3.10-rc1 or upper) */
@@ -680,13 +797,13 @@ static int select_tracing_timestamp(struct ras_events *ras)
 	return 0;
 }
 
-static int add_event_handler(struct ras_events *ras, struct pevent *pevent,
-			     unsigned page_size, char *group, char *event,
-			     pevent_event_handler_func func, char *filter_str, int id)
+static int add_event_handler(struct ras_events *ras, struct tep_handle *pevent,
+			     unsigned int page_size, char *group, char *event,
+			     tep_event_handler_func func, char *filter_str, int id)
 {
 	int fd, size, rc;
 	char *page, fname[MAX_PATH + 1];
-	struct event_filter * filter = NULL;
+	struct tep_event_filter *filter = NULL;
 
 	snprintf(fname, sizeof(fname), "events/%s/%s/format", group, event);
 
@@ -716,15 +833,15 @@ static int add_event_handler(struct ras_events *ras, struct pevent *pevent,
 	}
 
 	/* Registers the special event handlers */
-	rc = pevent_register_event_handler(pevent, -1, group, event, func, ras);
-	if (rc == PEVENT_ERRNO__MEM_ALLOC_FAILED) {
+	rc = tep_register_event_handler(pevent, -1, group, event, func, ras);
+	if (rc == TEP_ERRNO__MEM_ALLOC_FAILED) {
 		log(TERM, LOG_ERR, "Can't register event handler for %s:%s\n",
 		    group, event);
 		free(page);
 		return EINVAL;
 	}
 
-	rc = pevent_parse_event(pevent, page, size, group);
+	rc = tep_parse_event(pevent, page, size, group);
 	if (rc) {
 		log(TERM, LOG_ERR, "Can't parse event %s:%s\n", group, event);
 		free(page);
@@ -732,20 +849,21 @@ static int add_event_handler(struct ras_events *ras, struct pevent *pevent,
 	}
 
 	if (filter_str) {
-		char *error;
+		char error[255];
 
-		filter = pevent_filter_alloc(pevent);
+		filter = tep_filter_alloc(pevent);
 		if (!filter) {
 			log(TERM, LOG_ERR,
 			    "Failed to allocate filter for %s/%s.\n", group, event);
 			free(page);
 			return EINVAL;
 		}
-		rc = pevent_filter_add_filter_str(filter, filter_str, &error);
-		if (rc) {
+		rc = tep_filter_add_filter_str(filter, filter_str);
+		if (rc < 0) {
+			tep_filter_strerror(filter, rc, error, sizeof(error));
 			log(TERM, LOG_ERR,
 			    "Failed to install filter for %s/%s: %s\n", group, event, error);
-			pevent_filter_free(filter);
+			tep_filter_free(filter);
 			free(page);
 			return rc;
 		}
@@ -753,6 +871,12 @@ static int add_event_handler(struct ras_events *ras, struct pevent *pevent,
 
 	ras->filters[id] = filter;
 
+	if (is_disabled_event(group, event)) {
+		log(ALL, LOG_INFO, "Disabled %s:%s tracing from config\n",
+		    group, event);
+		return -EINVAL;
+	}
+
 	/* Enable RAS events */
 	rc = __toggle_ras_mc_event(ras, group, event, 1);
 	free(page);
@@ -763,6 +887,8 @@ static int add_event_handler(struct ras_events *ras, struct pevent *pevent,
 		return EINVAL;
 	}
 
+	setup_event_trigger(event);
+
 	log(ALL, LOG_INFO, "Enabled event %s:%s\n", group, event);
 
 	return 0;
@@ -772,8 +898,8 @@ int handle_ras_events(int record_events)
 {
 	int rc, page_size, i;
 	int num_events = 0;
-	unsigned cpus;
-	struct pevent *pevent = NULL;
+	unsigned int cpus;
+	struct tep_handle *pevent = NULL;
 	struct pthread_data *data = NULL;
 	struct ras_events *ras = NULL;
 #ifdef HAVE_DEVLINK
@@ -798,7 +924,7 @@ int handle_ras_events(int record_events)
 		goto err;
 	}
 
-	pevent = pevent_alloc();
+	pevent = tep_alloc();
 	if (!pevent) {
 		log(TERM, LOG_ERR, "Can't allocate pevent\n");
 		rc = errno;
@@ -820,7 +946,7 @@ int handle_ras_events(int record_events)
 			       ras_mc_event_handler, NULL, MC_EVENT);
 	if (!rc)
 		num_events++;
-	else
+	else if (rc != -EINVAL)
 		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
 		    "ras", "mc_event");
 
@@ -829,7 +955,7 @@ int handle_ras_events(int record_events)
 			       ras_aer_event_handler, NULL, AER_EVENT);
 	if (!rc)
 		num_events++;
-	else
+	else if (rc != -EINVAL)
 		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
 		    "ras", "aer_event");
 #endif
@@ -839,7 +965,7 @@ int handle_ras_events(int record_events)
 			       ras_non_standard_event_handler, NULL, NON_STANDARD_EVENT);
 	if (!rc)
 		num_events++;
-	else
+	else if (rc != -EINVAL)
 		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
 		    "ras", "non_standard_event");
 #endif
@@ -849,16 +975,20 @@ int handle_ras_events(int record_events)
 			       ras_arm_event_handler, NULL, ARM_EVENT);
 	if (!rc)
 		num_events++;
-	else
+	else if (rc != -EINVAL)
 		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
 		    "ras", "arm_event");
 #endif
 
 	cpus = get_num_cpus(ras);
 
+#ifdef HAVE_CPU_FAULT_ISOLATION
+	ras_cpu_isolation_init(cpus);
+#endif
+
 #ifdef HAVE_MCE
 	rc = register_mce_handler(ras, cpus);
-	if (rc)
+	if (rc && rc != -ENOENT)
 		log(ALL, LOG_INFO, "Can't register mce handler\n");
 	if (ras->mce_priv) {
 		rc = add_event_handler(ras, pevent, page_size,
@@ -879,7 +1009,7 @@ int handle_ras_events(int record_events)
 		/* tell kernel we are listening, so don't printk to console */
 		(void)open("/sys/kernel/debug/ras/daemon_active", 0);
 		num_events++;
-	} else
+	} else if (rc != -EINVAL)
 		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
 		    "ras", "extlog_mem_event");
 #endif
@@ -896,12 +1026,22 @@ int handle_ras_events(int record_events)
 			       ras_devlink_event_handler, filter_str, DEVLINK_EVENT);
 	if (!rc)
 		num_events++;
-	else
+	else if (rc != -EINVAL)
 		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
 		    "devlink", "devlink_health_report");
 #endif
 
 #ifdef HAVE_DISKERROR
+#ifdef HAVE_BLK_RQ_ERROR
+	rc = add_event_handler(ras, pevent, page_size, "block",
+			       "block_rq_error", ras_diskerror_event_handler,
+				NULL, DISKERROR_EVENT);
+	if (!rc)
+		num_events++;
+	else if (rc != -EINVAL)
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "block", "block_rq_error");
+#else
 	rc = filter_ras_mc_event(ras, "block", "block_rq_complete", "error != 0");
 	if (!rc) {
 		rc = add_event_handler(ras, pevent, page_size, "block",
@@ -909,22 +1049,89 @@ int handle_ras_events(int record_events)
 					NULL, DISKERROR_EVENT);
 		if (!rc)
 			num_events++;
-		else
+		else if (rc != -EINVAL)
 			log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
 			    "block", "block_rq_complete");
 	}
 #endif
+#endif
 
 #ifdef HAVE_MEMORY_FAILURE
 	rc = add_event_handler(ras, pevent, page_size, "ras", "memory_failure_event",
 			       ras_memory_failure_event_handler, NULL, MF_EVENT);
 	if (!rc)
 		num_events++;
-	else
+	else if (rc != -EINVAL)
 		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
 		    "ras", "memory_failure_event");
 #endif
 
+#ifdef HAVE_CXL
+	rc = add_event_handler(ras, pevent, page_size, "cxl", "cxl_poison",
+			       ras_cxl_poison_event_handler, NULL, CXL_POISON_EVENT);
+	if (!rc)
+		num_events++;
+	else if (rc != -EINVAL)
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "cxl", "cxl_poison");
+
+	rc = add_event_handler(ras, pevent, page_size, "cxl", "cxl_aer_uncorrectable_error",
+			       ras_cxl_aer_ue_event_handler, NULL, CXL_AER_UE_EVENT);
+	if (!rc)
+		num_events++;
+	else if (rc != -EINVAL)
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "cxl", "cxl_aer_uncorrectable_error");
+
+	rc = add_event_handler(ras, pevent, page_size, "cxl", "cxl_aer_correctable_error",
+			       ras_cxl_aer_ce_event_handler, NULL, CXL_AER_CE_EVENT);
+	if (!rc)
+		num_events++;
+	else if (rc != -EINVAL)
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "cxl", "cxl_aer_correctable_error");
+
+	rc = add_event_handler(ras, pevent, page_size, "cxl", "cxl_overflow",
+			       ras_cxl_overflow_event_handler, NULL, CXL_OVERFLOW_EVENT);
+	if (!rc)
+		num_events++;
+	else if (rc != -EINVAL)
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "cxl", "cxl_overflow");
+
+	rc = add_event_handler(ras, pevent, page_size, "cxl", "cxl_generic_event",
+			       ras_cxl_generic_event_handler, NULL, CXL_GENERIC_EVENT);
+	if (!rc)
+		num_events++;
+	else if (rc != -EINVAL)
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "cxl", "cxl_generic_event");
+
+	rc = add_event_handler(ras, pevent, page_size, "cxl", "cxl_general_media",
+			       ras_cxl_general_media_event_handler, NULL, CXL_GENERAL_MEDIA_EVENT);
+	if (!rc)
+		num_events++;
+	else if (rc != -EINVAL)
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "cxl", "cxl_general_media");
+
+	rc = add_event_handler(ras, pevent, page_size, "cxl", "cxl_dram",
+			       ras_cxl_dram_event_handler, NULL, CXL_DRAM_EVENT);
+	if (!rc)
+		num_events++;
+	else if (rc != -EINVAL)
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "cxl", "cxl_dram");
+
+	rc = add_event_handler(ras, pevent, page_size, "cxl", "cxl_memory_module",
+			       ras_cxl_memory_module_event_handler, NULL, CXL_MEMORY_MODULE_EVENT);
+	if (!rc)
+		num_events++;
+	else if (rc != -EINVAL)
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "cxl", "memory_module");
+#endif
+
 	if (!num_events) {
 		log(ALL, LOG_INFO,
 		    "Failed to trace all supported RAS events. Aborting.\n");
@@ -936,7 +1143,6 @@ int handle_ras_events(int record_events)
 	if (!data)
 		goto err;
 
-
 	for (i = 0; i < cpus; i++) {
 		data[i].ras = ras;
 		data[i].cpu = i;
@@ -945,18 +1151,25 @@ int handle_ras_events(int record_events)
 
 	/* Poll doesn't work on this kernel. Fallback to pthread way */
 	if (rc == -255) {
+		if (pthread_mutex_init(&ras->db_lock, NULL) != 0) {
+			log(SYSLOG, LOG_INFO, "sqlite db lock init has failed\n");
+			goto err;
+		}
+
 		log(SYSLOG, LOG_INFO,
-		"Opening one thread per cpu (%d threads)\n", cpus);
+		    "Opening one thread per cpu (%d threads)\n", cpus);
 		for (i = 0; i < cpus; i++) {
 			rc = pthread_create(&data[i].thread, NULL,
-					handle_ras_events_cpu,
+					    handle_ras_events_cpu,
 					(void *)&data[i]);
 			if (rc) {
 				log(SYSLOG, LOG_INFO,
-				"Failed to create thread for cpu %d. Aborting.\n",
+				    "Failed to create thread for cpu %d. Aborting.\n",
 				i);
 				while (--i)
 					pthread_cancel(data[i].thread);
+
+				pthread_mutex_destroy(&ras->db_lock);
 				goto err;
 			}
 		}
@@ -964,6 +1177,7 @@ int handle_ras_events(int record_events)
 		/* Wait for all threads to complete */
 		for (i = 0; i < cpus; i++)
 			pthread_join(data[i].thread, NULL);
+		pthread_mutex_destroy(&ras->db_lock);
 	}
 
 	log(SYSLOG, LOG_INFO, "Huh! something got wrong. Aborting.\n");
@@ -973,15 +1187,17 @@ err:
 		free(data);
 
 	if (pevent)
-		pevent_free(pevent);
+		tep_free(pevent);
 
 	if (ras) {
 		for (i = 0; i < NR_EVENTS; i++) {
 			if (ras->filters[i])
-				pevent_filter_free(ras->filters[i]);
+				tep_filter_free(ras->filters[i]);
 		}
 		free(ras);
 	}
-
+#ifdef HAVE_CPU_FAULT_ISOLATION
+	cpu_infos_free();
+#endif
 	return rc;
 }
