@@ -20,10 +20,12 @@
 #define __RAS_MC_HANDLER_H
 
 #include "ras-events.h"
-#include "libtrace/event-parse.h"
+#include <traceevent/event-parse.h>
+
+void mc_event_trigger_setup(void);
 
 int ras_mc_event_handler(struct trace_seq *s,
-			 struct pevent_record *record,
-			 struct event_format *event, void *context);
+			 struct tep_record *record,
+			 struct tep_event *event, void *context);
 
 #endif
