@@ -16,10 +16,11 @@
 #define __RAS_MEMORY_FAILURE_HANDLER_H
 
 #include "ras-events.h"
-#include "libtrace/event-parse.h"
+#include <traceevent/event-parse.h>
 
+void mem_fail_event_trigger_setup(void);
 int ras_memory_failure_event_handler(struct trace_seq *s,
-				     struct pevent_record *record,
-				     struct event_format *event, void *context);
+				     struct tep_record *record,
+				     struct tep_event *event, void *context);
 
 #endif
