@@ -16,7 +16,7 @@
 #include <stdbool.h>
 #include <string.h>
 #include <unistd.h>
-#include "libtrace/kbuffer.h"
+#include <traceevent/kbuffer.h>
 #include "ras-non-standard-handler.h"
 #include "ras-record.h"
 #include "ras-logger.h"
@@ -24,8 +24,9 @@
 
 static struct  ras_ns_ev_decoder *ras_ns_ev_dec_list;
 
-void print_le_hex(struct trace_seq *s, const uint8_t *buf, int index) {
-	trace_seq_printf(s, "%02x%02x%02x%02x", buf[index+3], buf[index+2], buf[index+1], buf[index]);
+void print_le_hex(struct trace_seq *s, const uint8_t *buf, int index)
+{
+	trace_seq_printf(s, "%02x%02x%02x%02x", buf[index + 3], buf[index + 2], buf[index + 1], buf[index]);
 }
 
 static char *uuid_le(const char *uu)
@@ -33,10 +34,10 @@ static char *uuid_le(const char *uu)
 	static char uuid[sizeof("xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx")];
 	char *p = uuid;
 	int i;
-	static const unsigned char le[16] = {3,2,1,0,5,4,7,6,8,9,10,11,12,13,14,15};
+	static const unsigned char le[16] = {3, 2, 1, 0, 5, 4, 7, 6, 8, 9, 10, 11, 12, 13, 14, 15};
 
 	for (i = 0; i < 16; i++) {
-		p += sprintf(p, "%.2x", (unsigned char) uu[le[i]]);
+		p += sprintf(p, "%.2x", (unsigned char)uu[le[i]]);
 		switch (i) {
 		case 3:
 		case 5:
@@ -52,20 +53,6 @@ static char *uuid_le(const char *uu)
 	return uuid;
 }
 
-static int uuid_le_cmp(const char *sec_type, const char *uuid2)
-{
-	static char uuid1[32];
-	char *p = uuid1;
-	int i;
-	static const unsigned char le[16] = {
-			3, 2, 1, 0, 5, 4, 7, 6, 8, 9, 10, 11, 12, 13, 14, 15};
-
-	for (i = 0; i < 16; i++)
-		p += sprintf(p, "%.2x", (unsigned char) sec_type[le[i]]);
-	*p = 0;
-	return strncmp(uuid1, uuid2, 32);
-}
-
 int register_ns_ev_decoder(struct ras_ns_ev_decoder *ns_ev_decoder)
 {
 	struct ras_ns_ev_decoder *list;
@@ -79,6 +66,7 @@ int register_ns_ev_decoder(struct ras_ns_ev_decoder *ns_ev_decoder)
 #endif
 	if (!ras_ns_ev_dec_list) {
 		ras_ns_ev_dec_list = ns_ev_decoder;
+		ras_ns_ev_dec_list->ref_count = 0;
 	} else {
 		list = ras_ns_ev_dec_list;
 		while (list->next)
@@ -89,6 +77,34 @@ int register_ns_ev_decoder(struct ras_ns_ev_decoder *ns_ev_decoder)
 	return 0;
 }
 
+int ras_ns_add_vendor_tables(struct ras_events *ras)
+{
+	struct ras_ns_ev_decoder *ns_ev_decoder;
+	int error = 0;
+
+#ifdef HAVE_SQLITE3
+	if (!ras)
+		return -1;
+
+	ns_ev_decoder = ras_ns_ev_dec_list;
+	if (ras_ns_ev_dec_list)
+		ras_ns_ev_dec_list->ref_count++;
+	while (ns_ev_decoder) {
+		if (ns_ev_decoder->add_table && !ns_ev_decoder->stmt_dec_record) {
+			error = ns_ev_decoder->add_table(ras, ns_ev_decoder);
+			if (error)
+				break;
+		}
+		ns_ev_decoder = ns_ev_decoder->next;
+	}
+
+	if (error)
+		return -1;
+#endif
+
+	return 0;
+}
+
 static int find_ns_ev_decoder(const char *sec_type, struct ras_ns_ev_decoder **p_ns_ev_dec)
 {
 	struct ras_ns_ev_decoder *ns_ev_decoder;
@@ -96,7 +112,7 @@ static int find_ns_ev_decoder(const char *sec_type, struct ras_ns_ev_decoder **p
 
 	ns_ev_decoder = ras_ns_ev_dec_list;
 	while (ns_ev_decoder) {
-		if (uuid_le_cmp(sec_type, ns_ev_decoder->sec_type) == 0) {
+		if (strcmp(uuid_le(sec_type), ns_ev_decoder->sec_type) == 0) {
 			*p_ns_ev_dec = ns_ev_decoder;
 			match  = 1;
 			break;
@@ -110,11 +126,21 @@ static int find_ns_ev_decoder(const char *sec_type, struct ras_ns_ev_decoder **p
 	return 0;
 }
 
-static void unregister_ns_ev_decoder(void)
+void ras_ns_finalize_vendor_tables(void)
 {
 #ifdef HAVE_SQLITE3
 	struct ras_ns_ev_decoder *ns_ev_decoder = ras_ns_ev_dec_list;
 
+	if (!ras_ns_ev_dec_list)
+		return;
+
+	if (ras_ns_ev_dec_list->ref_count > 0)
+		ras_ns_ev_dec_list->ref_count--;
+	else
+		return;
+	if (ras_ns_ev_dec_list->ref_count > 0)
+		return;
+
 	while (ns_ev_decoder) {
 		if (ns_ev_decoder->stmt_dec_record) {
 			ras_mc_finalize_vendor_table(ns_ev_decoder->stmt_dec_record);
@@ -122,13 +148,23 @@ static void unregister_ns_ev_decoder(void)
 		}
 		ns_ev_decoder = ns_ev_decoder->next;
 	}
+#endif
+}
+
+static void unregister_ns_ev_decoder(void)
+{
+#ifdef HAVE_SQLITE3
+	if (!ras_ns_ev_dec_list)
+		return;
+	ras_ns_ev_dec_list->ref_count = 1;
+	ras_ns_finalize_vendor_tables();
 #endif
 	ras_ns_ev_dec_list = NULL;
 }
 
 int ras_non_standard_event_handler(struct trace_seq *s,
-			 struct pevent_record *record,
-			 struct event_format *event, void *context)
+				   struct tep_record *record,
+				   struct tep_event *event, void *context)
 {
 	int len, i, line_count;
 	unsigned long long val;
@@ -148,7 +184,7 @@ int ras_non_standard_event_handler(struct trace_seq *s,
 	 */
 
 	if (ras->use_uptime)
-		now = record->ts/user_hz + ras->uptime_diff;
+		now = record->ts / user_hz + ras->uptime_diff;
 	else
 		now = time(NULL);
 
@@ -158,7 +194,7 @@ int ras_non_standard_event_handler(struct trace_seq *s,
 			 "%Y-%m-%d %H:%M:%S %z", tm);
 	trace_seq_printf(s, "%s ", ev.timestamp);
 
-	if (pevent_get_field_val(s, event, "sev", record, &val, 1) < 0)
+	if (tep_get_field_val(s, event, "sev", record, &val, 1) < 0)
 		return -1;
 	switch (val) {
 	case GHES_SEV_NO:
@@ -174,34 +210,33 @@ int ras_non_standard_event_handler(struct trace_seq *s,
 	case GHES_SEV_PANIC:
 		ev.severity = "Fatal";
 	}
-	trace_seq_printf(s, "\n %s", ev.severity);
+	trace_seq_printf(s, " %s", ev.severity);
 
-	ev.sec_type = pevent_get_field_raw(s, event, "sec_type",
-					   record, &len, 1);
-	if(!ev.sec_type)
+	ev.sec_type = tep_get_field_raw(s, event, "sec_type",
+					record, &len, 1);
+	if (!ev.sec_type)
 		return -1;
 	if (strcmp(uuid_le(ev.sec_type),
 		   "e8ed898d-df16-43cc-8ecc-54f060ef157f") == 0)
-		trace_seq_printf(s, "\n section type: %s",
-		"Ampere Specific Error\n");
+		trace_seq_printf(s, " section type: %s",
+				 "Ampere Specific Error");
 	else
-		trace_seq_printf(s, "\n section type: %s",
+		trace_seq_printf(s, " section type: %s",
 				 uuid_le(ev.sec_type));
-	ev.fru_text = pevent_get_field_raw(s, event, "fru_text",
-						record, &len, 1);
-	ev.fru_id = pevent_get_field_raw(s, event, "fru_id",
-						record, &len, 1);
+	ev.fru_text = tep_get_field_raw(s, event, "fru_text",
+					record, &len, 1);
+	ev.fru_id = tep_get_field_raw(s, event, "fru_id",
+				      record, &len, 1);
 	trace_seq_printf(s, " fru text: %s fru id: %s ",
-				ev.fru_text,
-				uuid_le(ev.fru_id));
+			 ev.fru_text, uuid_le(ev.fru_id));
 
-	if (pevent_get_field_val(s, event, "len", record, &val, 1) < 0)
+	if (tep_get_field_val(s, event, "len", record, &val, 1) < 0)
 		return -1;
 	ev.length = val;
-	trace_seq_printf(s, "\n length: %d\n", ev.length);
+	trace_seq_printf(s, " length: %d", ev.length);
 
-	ev.error = pevent_get_field_raw(s, event, "buf", record, &len, 1);
-	if(!ev.error)
+	ev.error = tep_get_field_raw(s, event, "buf", record, &len, 1);
+	if (!ev.error)
 		return -1;
 
 	if (!find_ns_ev_decoder(ev.sec_type, &ns_ev_decoder)) {
